import { any, hasIn, is, path, pipe, split } from 'ramda';
var INDENTATION = 12;
var prop = pipe(split(/[[\].]/), path);
export var isListElement = function (val) {
    return is(Object, val) && !prop('key')(val);
};
export var isDictionaryElement = function (value) {
    return is(Object, value) &&
        hasIn('key', value) &&
        (is(String, prop('key')(value)) || is(Number, prop('key')(value)));
};
export var isMetaList = function (value) {
    return is(Array, value) && any(isListElement, value);
};
export var isMetaDictionary = function (value) {
    return is(Array, value) && any(isDictionaryElement, value);
};
var stringify = function (space, val, padding) {
    var indentation = ' '.repeat(padding);
    return JSON.stringify(val, null, space)
        .replace(/\\n/g, '\n')
        .replace(/}$/, indentation + '}')
        .replace(/]$/, indentation + ']');
};
var isJsonString = function (value) {
    try {
        var parsedValue = JSON.parse(value);
        return is(Object, parsedValue);
    }
    catch (e) {
        return false;
    }
};
var isBooleanString = function (value) {
    return value === 'true' || value === 'false';
};
var getQuotedString = function (value) {
    return isBooleanString(value) ? value : "\"" + value + "\"";
};
export function normalizeIssueVariable(keyValue, value, padding, indentation, withoutQuotes) {
    var _a;
    if (indentation === void 0) { indentation = INDENTATION; }
    if (is(String, value)) {
        if (isJsonString(String(value))) {
            return stringify(indentation, JSON.parse(String(value)), padding);
        }
        var elementPadding_1 = ' '.repeat((withoutQuotes ? 3 : 5) + padding + keyValue.length);
        var formattedValue = value
            .split(/\n/)
            .map(function (item, index) { return (index === 0 ? item : "" + elementPadding_1 + item); })
            .join('\n');
        return getQuotedString(formattedValue);
    }
    if (isMetaList(value) || isMetaDictionary(value)) {
        var items = value.filter(Boolean).map(function (item) {
            if (is(String, item)) {
                return isJsonString(item) ? JSON.parse(item) : item;
            }
            var key = item.key, value = item.value;
            if (key) {
                return key + ": " + value;
            }
            return isJsonString(String(value)) ? JSON.parse(String(value)) : value;
        });
        return stringify(indentation, items, padding);
    }
    if (isDictionaryElement(value)) {
        return stringify(keyValue.length, (_a = {}, _a[value.key] = value.value, _a), padding);
    }
    if (isListElement(value)) {
        if (is(Array, value.value)) {
            return stringify(indentation, value.value, padding);
        }
        var listValue = String(value.value);
        return isJsonString(listValue)
            ? stringify(indentation, JSON.parse(listValue), padding)
            : getQuotedString(listValue);
    }
    return String(value);
}
